import os
import datetime
from ast import literal_eval

import boto3

from cloud_governance.common.clouds.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.clouds.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.clouds.aws.ec2.ec2_operations import EC2Operations
from cloud_governance.common.clouds.aws.s3.s3_operations import S3Operations
from cloud_governance.common.ldap.ldap_search import LdapSearch
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix
from cloud_governance.policy.aws.zombie_cluster_resource import ZombieClusterResources


class NonClusterZombiePolicy:

    def __init__(self):
        self._account = os.environ.get('account', '')
        self._dry_run = os.environ.get('dry_run', 'yes')
        self._region = os.environ.get('AWS_DEFAULT_REGION', 'us-east-2')
        self._policy = os.environ.get('policy', '')
        self._policy_output = os.environ.get('policy_output', '')
        self._ec2_client = boto3.client('ec2', region_name=self._region)
        self._ec2_operations = EC2Operations(region=self._region)
        self._s3_client = boto3.client('s3')
        self._iam_operations = IAMOperations()
        self._iam_client = boto3.client('iam')
        self._cluster_prefix = 'kubernetes.io/cluster'
        self._zombie_cluster = ZombieClusterResources(cluster_prefix=self._cluster_prefix)
        self._s3operations = S3Operations(region_name=self._region)
        self._cloudtrail = CloudTrailOperations(region_name=self._region)
        self._special_user_mails = os.environ.get('special_user_mails', '{}')
        self._account_admin = os.environ.get('account_admin', '')
        self._mail = Postfix()
        self._mail_description = MailMessage()
        self.__ldap_host_name = os.environ.get('LDAP_HOST_NAME', '')
        self._ldap = LdapSearch(ldap_host_name=self.__ldap_host_name)

    def _literal_eval(self, data: any):
        tags = {}
        if data:
            tags = literal_eval(data)
        return tags

    def _check_live_cluster_tag(self, tags: list, active_clusters: list):
        """
        This method returns True if it is live cluster tag is active False not
        @param tags:
        @return:
        """
        if tags:
            for tag in tags:
                if tag.get('Key').startswith(self._cluster_prefix):
                    return tag.get('Key') in active_clusters
        return False

    def _get_tag_name_from_tags(self, tags: list, tag_name: str = 'Name'):
        """
        This method returns the tag_name from resource tags
        @param tags:
        @param tag_name:
        @return:
        """
        if tags:
            for tag in tags:
                if tag.get('Key').strip().lower() == tag_name.lower():
                    return tag.get('Value').strip()
            return 'NA'
        return 'NA'

    def _calculate_days(self, create_date: datetime):
        """
        This method return the days
        @return:
        """
        today = datetime.date.today()
        days = today - create_date.date()
        return days.days

    def _get_ami_ids(self):
        """
        This method returns all image ids
        @return:
        """
        images = self._ec2_operations.get_images()
        image_ids = []
        for image in images:
            image_ids.append(image.get('ImageId'))
        return image_ids

    def _beautify_upload_data(self, upload_resource_data: list):
        """
        This method beautify the data to upload to elasticsearch
        @param upload_resource_data:
        @return:
        """
        upload_data = []
        for resource_data in upload_resource_data:
            if isinstance(resource_data, list):
                resource_str = ''
                for resource in resource_data[:-1]:
                    resource_str += f'{resource} | '
                resource_str += resource_data[-1]
                upload_data.append(resource_str)
            else:
                upload_data.append(resource_data)
        return upload_data

    def _get_resource_username(self, create_date: datetime, resource_id: str, resource_type: str):
        """
        Get Username from the cloudtrail
        @param create_date:
        @param resource_id:
        @param resource_type:
        @return:
        """
        return self._cloudtrail.get_username_by_instance_id_and_time(start_time=create_date, resource_id=resource_id, resource_type=resource_type)

    def _get_policy_value(self, tags: list):
        """
        This method beautify the value
        @param tags:
        @return:
        """
        policy_value = self._get_tag_name_from_tags(tags=tags, tag_name='Policy').strip()
        return policy_value.replace('_', '').replace('-', '').upper()




import typeguard

from cloud_governance.common.logger.init_logger import logger
from cloud_governance.policy.aws.zombie_cluster_resource import ZombieClusterResources


@typeguard.typechecked
def __get_resource_list(region, delete: bool = False, resource: str = '', cluster_tag: str = '',
                        resource_name: str = '', service_type: str = ' '):
    zombie_cluster_resources = ZombieClusterResources(cluster_prefix='kubernetes.io/cluster/', delete=delete,
                                                      region=region, cluster_tag=cluster_tag,
                                                      resource_name=resource_name)
    zombie_cluster_resources_dict = {'zombie_cluster_volume': zombie_cluster_resources.zombie_cluster_volume,
                                     'zombie_cluster_ami': zombie_cluster_resources.zombie_cluster_ami,
                                     'zombie_cluster_snapshot': zombie_cluster_resources.zombie_cluster_snapshot,
                                     'zombie_cluster_security_group': zombie_cluster_resources.zombie_cluster_security_group,
                                     'zombie_cluster_elastic_ip': zombie_cluster_resources.zombie_cluster_elastic_ip,
                                     'zombie_cluster_network_interface': zombie_cluster_resources.zombie_cluster_network_interface,
                                     'zombie_cluster_load_balancer': zombie_cluster_resources.zombie_cluster_load_balancer,
                                     'zombie_cluster_load_balancer_v2': zombie_cluster_resources.zombie_cluster_load_balancer_v2,
                                     'zombie_cluster_vpc': zombie_cluster_resources.zombie_cluster_vpc,
                                     'zombie_cluster_subnet': zombie_cluster_resources.zombie_cluster_subnet,
                                     'zombie_cluster_route_table': zombie_cluster_resources.zombie_cluster_route_table,
                                     'zombie_cluster_internet_gateway': zombie_cluster_resources.zombie_cluster_internet_gateway,
                                     'zombie_cluster_dhcp_option': zombie_cluster_resources.zombie_cluster_dhcp_option,
                                     'zombie_cluster_vpc_endpoint': zombie_cluster_resources.zombie_cluster_vpc_endpoint,
                                     'zombie_cluster_nat_gateway': zombie_cluster_resources.zombie_cluster_nat_gateway,
                                     'zombie_cluster_network_acl': zombie_cluster_resources.zombie_cluster_network_acl,
                                     'zombie_cluster_role': zombie_cluster_resources.zombie_cluster_role,
                                     'zombie_cluster_user': zombie_cluster_resources.zombie_cluster_user,
                                     'zombie_cluster_s3_bucket': zombie_cluster_resources.zombie_cluster_s3_bucket}
    ec2_zombie_resource_services = [zombie_cluster_resources.zombie_cluster_ami,
                                    zombie_cluster_resources.zombie_cluster_volume,
                                    zombie_cluster_resources.zombie_cluster_load_balancer,
                                    zombie_cluster_resources.zombie_cluster_load_balancer_v2,
                                    zombie_cluster_resources.zombie_cluster_snapshot,
                                    zombie_cluster_resources.zombie_cluster_vpc_endpoint,
                                    zombie_cluster_resources.zombie_cluster_dhcp_option,
                                    zombie_cluster_resources.zombie_cluster_route_table,
                                    zombie_cluster_resources.zombie_cluster_security_group,
                                    zombie_cluster_resources.zombie_cluster_nat_gateway,
                                    zombie_cluster_resources.zombie_cluster_network_acl,
                                    zombie_cluster_resources.zombie_cluster_network_interface,
                                    zombie_cluster_resources.zombie_cluster_elastic_ip,
                                    zombie_cluster_resources.zombie_cluster_internet_gateway,
                                    zombie_cluster_resources.zombie_cluster_subnet,
                                    zombie_cluster_resources.zombie_cluster_vpc]
    iam_zombie_resource_services = [zombie_cluster_resources.zombie_cluster_role,
                                    zombie_cluster_resources.zombie_cluster_user]
    s3_zombie_resource_services = [zombie_cluster_resources.zombie_cluster_s3_bucket]
    scan_func_resource_list = []
    delete_func_resource_list = []
    if resource:
        scan_func_resource_list = [zombie_cluster_resources_dict[resource]]
        delete_func_resource_list = [zombie_cluster_resources_dict[resource]]
    else:
        if service_type:
            if service_type == 'ec2_zombie_resource_service':
                scan_func_resource_list.extend(ec2_zombie_resource_services)
                delete_func_resource_list.extend(ec2_zombie_resource_services)
            elif service_type == 'iam_zombie_resource_service':
                scan_func_resource_list.extend(iam_zombie_resource_services)
                delete_func_resource_list.extend(iam_zombie_resource_services)
            elif service_type == 's3_zombie_resource_service':
                scan_func_resource_list.extend(s3_zombie_resource_services)
                delete_func_resource_list.extend(s3_zombie_resource_services)
        else:
            scan_func_resource_list = ec2_zombie_resource_services + iam_zombie_resource_services + s3_zombie_resource_services
            delete_func_resource_list = ec2_zombie_resource_services + iam_zombie_resource_services + s3_zombie_resource_services
    if delete:
        return delete_func_resource_list
    else:
        return scan_func_resource_list


@typeguard.typechecked
def zombie_cluster_resource(delete: bool = False, region: str = 'us-east-2', resource: str = '', cluster_tag: str = '',
                            resource_name: str = '', service_type: str = ''):
    """
    This method return zombie cluster resources,
    How its works? if not exist an instance cluster, the resource is zombie
    if delete true it will delete the zombie resource
    :return: list of zombie resources
    """
    zombie_result = {}
    all_cluster_data = []

    if delete:
        action = 'Delete'
        if resource:
            func_resource_list = __get_resource_list(region, delete, resource, cluster_tag, resource_name, service_type)
        else:
            func_resource_list = __get_resource_list(region, delete, resource, cluster_tag, resource_name, service_type)
        logger.info(
            "Skip Deleting the following resource due to Dependencies:\n zombie_cluster_security_group, zombie_cluster_vpc, zombie_cluster_route_table, zombie_cluster_internet_gateway, zombie_cluster_dhcp_option")
    else:
        action = 'Scan'
        if resource:
            func_resource_list = __get_resource_list(region, delete, resource, cluster_tag, resource_name, service_type)
        elif resource_name:
            func_resource_list = __get_resource_list(region, delete, resource, cluster_tag, resource_name, service_type)
        else:
            func_resource_list = __get_resource_list(region, delete, resource, cluster_tag, resource_name, service_type)
    logger.info(f'{action} {len(func_resource_list)} cluster zombies resources in region {region}:')
    for func in func_resource_list:
        resource_data = func()
        if resource_data:
            resource_data_list = sorted(set(resource_data.values()))
            resource_data_list = [func.__name__.replace("zombie_cluster_", "") + ':' + item for item in
                                  resource_data_list]
            logger.info(f'key: {func.__name__}, count: {len(resource_data)}, data: {resource_data_list}')
            zombie_result[func.__name__] = {'count': len(resource_data), 'data': resource_data_list}
            all_cluster_data.extend(resource_data_list)
    zombie_result['all_cluster_data'] = {'count': len(set(all_cluster_data)), 'data': set(sorted(all_cluster_data))}
    return zombie_result



import operator
from operator import ge

from botocore.exceptions import ClientError

from cloud_governance.common.logger.init_logger import logger
from cloud_governance.aws.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy


class EmptyBuckets(NonClusterZombiePolicy):

    BUCKET_DAYS = 7

    def __init__(self):
        super().__init__()

    def run(self):
        """
        This method return all Empty buckets and delete if dry_run no
        @return:
        """
        return self.__delete_empty_bucket(bucket_days=self.BUCKET_DAYS, sign=ge)

    def __delete_empty_bucket(self, bucket_days: int, sign: operator = ge):
        """
        This method delete the empty bucket morethan 30 days
        @param bucket_days:
        @param sign:
        @return:
        """
        zombie_buckets = {}
        zombie_buckets_data = []
        buckets = self._s3_client.list_buckets()['Buckets']
        active_clusters = self._zombie_cluster.all_cluster_instance()
        for bucket in buckets:
            age = self._calculate_days(create_date=bucket.get('CreationDate'))
            if sign(age, bucket_days):
                try:
                    bucket_data = self._s3_client.list_objects_v2(Bucket=bucket.get('Name'))
                    if not bucket_data.get('Contents'):
                        try:
                            bucket_tags = self._s3_client.get_bucket_tagging(Bucket=bucket.get('Name'))
                            tags = bucket_tags.get('TagSet')
                        except ClientError:
                            tags = []
                        if not self._check_live_cluster_tag(tags, active_clusters.values()):
                            zombie_buckets[bucket.get('Name')] = tags
                            zombie_buckets_data.append([bucket.get('Name'),
                                                        str(bucket.get('CreationDate')),
                                                        str(age),
                                                        self._get_policy_value(tags=tags)
                                                        ])
                except Exception as err:
                    logger.info(f'{err}, {bucket.get("Name")}')
        if self._dry_run == 'no':
            for zombie_bucket, tags in zombie_buckets.items():
                if self._get_policy_value(tags=tags) != 'NOTDELETE':
                    try:
                        self._s3_client.delete_bucket(Bucket=zombie_bucket)
                        logger.info(f'Bucket is deleted {zombie_bucket}')
                    except Exception as err:
                        logger.info(f'Exception raised: {err}: {zombie_bucket}')
        return zombie_buckets_data



import csv
import os
from ast import literal_eval

import boto3
import pandas as pd

from cloud_governance.common.clouds.aws.iam.iam_operations import IAMOperations
from cloud_governance.common.clouds.aws.utils.utils import Utils
from cloud_governance.common.google_drive.google_drive_operations import GoogleDriveOperations
from cloud_governance.common.ldap.ldap_search import LdapSearch
from cloud_governance.common.logger.init_logger import logger
from cloud_governance.common.mails.mail_message import MailMessage
from cloud_governance.common.mails.postfix import Postfix


class TagUser:
    """
    Tag user in the AWS account
    """

    def __init__(self, file_name: str):
        self.iam_client = boto3.client('iam')
        self.get_detail_resource_list = Utils().get_details_resource_list
        self.IAMOperations = IAMOperations()
        self.file_name = file_name
        self.__SPREADSHEET_ID = os.environ.get('SPREADSHEET_ID', '')
        self.__ldap_host_name = os.environ.get('LDAP_HOST_NAME', '')
        self.__ldap = LdapSearch(ldap_host_name=self.__ldap_host_name)
        if self.__SPREADSHEET_ID:
            self.__google_drive_operations = GoogleDriveOperations()
            self.__sheet_name = os.environ.get('account', '')
            self.__mail = Postfix()
            self._special_user_mails = os.environ.get('special_user_mails', '{}')

    def __literal_eval(self):
        tags = {}
        if self._special_user_mails:
            tags = literal_eval(self._special_user_mails)
        return tags

    def __cluster_user(self, tags: list):
        """
        This method check the user is cluster or not
        @param tags:
        @return:
        """
        for tag in tags:
            if 'kubernetes.io/cluster' in tag.get('Key'):
                return True
        return False

    def __write_into_csv_file(self, tag_keys: list, tag_values: dict):
        """
        This method create a csv file and append data into it
        @param tag_keys:
        @param tag_values:
        @return:
        """
        with open(self.file_name, 'w') as file:
            file.write('Username, ')
            if 'Username' in tag_keys:
                tag_keys.remove('Username')
            for index, tag in enumerate(tag_keys):
                file.write(f'{tag}, ')
            file.write('\n')
            tag_keys = list(tag_keys)
            for key, values in tag_values.items():
                values = dict(sorted(values.items()))
                file.write(f'{key}, ')
                if 'Username' in values:
                    values.pop('Username')
                for tag_key in tag_keys:
                    if tag_key in values:
                        file.write(f'{values.get(tag_key)}, ')
                    else:
                        file.write(' , ')
                file.write('\n')

    def generate_user_csv(self):
        """
        This method generates the User csv
        @return:
        """
        users = self.get_detail_resource_list(func_name=self.iam_client.list_users, input_tag='Users',
                                          check_tag='Marker')
        tag_keys = set()
        tag_values = {}
        for user in users:
            user_name = user.get('UserName')
            if '-' not in user_name:
                user_tags = self.IAMOperations.get_user_tags(username=user_name)
                tag_values[user_name] = {}
                for tag in user_tags:
                    if not self.__cluster_user(tags=user_tags):
                        key = tag.get('Key')
                        if key == "Name":
                            key = 'Username'
                        value = tag.get('Value')
                        tag_keys.add(key)
                        tag_values[user_name][key] = value
                    else:
                        del tag_values[user_name]
                        break
        tag_keys = list(sorted(tag_keys))
        self.__write_into_csv_file(tag_keys=tag_keys, tag_values=tag_values)
        with open(self.file_name) as file:
            logger.info(file.read())

    def __filter_tags_user_tags(self, user_tags: list, append_tags: list):
        """
        This method filter the tad of user ad updated tags of user
        @param user_tags:
        @param append_tags:
        @return:
        """
        add_tags = []
        if user_tags:
            for append_tag in append_tags:
                found = False
                for user_tag in user_tags:
                    if user_tag.get('Key').strip() == append_tag.get('Key').strip():
                        if user_tag.get('Value').strip() == append_tag.get('Value').strip():
                            found = True
                if not found:
                    add_tags.append(append_tag)
        else:
            add_tags.extend(append_tags)
        return add_tags

    def __get_tag(self, key, value):
        """
        This method creates a pair of Key Value pair
        @param key:
        @param value:
        @return:
        """
        return {'Key': key, 'Value': value}

    def __get_json_data(self, header: list, rows: list):
        """
        This method convert data list into dictionary
        @param header:
        @param rows:
        @return:
        """
        tagging = {}
        for row in rows:
            username = row[0].strip()
            tagging[username] = []
            for i in range(1, len(row)):
                key = header[i].strip()
                value = row[i].strip().upper().replace('\'', ' ')
                if value:
                    tagging[username].append(self.__get_tag(key, value))
        return tagging

    def update_user_tags(self):
        """
        This method updates the user tags from the csv file
        @return:
        """
        count = 0
        updated_usernames = []
        with open(self.file_name) as file:
            csvreader = csv.reader(file)
            header = next(csvreader)
            rows = []
            for row in csvreader:
                rows.append(row)

            json_data = self.__get_json_data(header, rows)
            for key, tags in json_data.items():
                try:
                    user_tags = self.IAMOperations.get_user_tags(username=key)
                    tags.append({'Key': 'User', 'Value': key})
                    filter_tags = self.__filter_tags_user_tags(user_tags, tags)
                    if filter_tags:
                        self.iam_client.tag_user(UserName=key, Tags=filter_tags)
                        logger.info(f'Username :: {key} {filter_tags}')
                        updated_usernames.append(key)
                        count += 1
                except Exception as err:
                    logger.info(err)
        logger.info(f'Updated Tags of IAM Users = {count} :: Usernames {updated_usernames}')
        return count

    def __format_tags(self, username: str, headers: list):
        tags = {'User': username}
        user_tags = self.IAMOperations.get_user_tags(username=username)
        for user_tag in user_tags:
            if user_tag.get('Key') in headers:
                tags[user_tag.get('Key')] = user_tag.get('Value')
        return tags

    def delete_update_user_from_doc(self):
        """
        This method removes IAM user if not in the IAM list
        @return:
        """
        iam_file = pd.read_csv(self.file_name)
        iam_users = [user['UserName'] for user in self.IAMOperations.get_users()]
        csv_iam_users = list(iam_file['User'])
        for index, user in enumerate(csv_iam_users):
            if user not in iam_users:
                self.__google_drive_operations.delete_rows(spreadsheet_id=self.__SPREADSHEET_ID, sheet_name=self.__sheet_name, row_number=index+1)
                logger.info(f'removed user {user}')
        append_data = []
        for user in iam_users:
            if '-' not in user:
                if user not in csv_iam_users:
                    tags = self.__format_tags(username=user, headers=list(iam_file.columns))
                    df2 = pd.DataFrame.from_dict([tags])
                    iam_file = pd.concat([iam_file, df2], ignore_index=True)
                    iam_file = iam_file.fillna('')
                    append_data.append(list(iam_file.iloc[-1]))
                    if len(tags) < len(list(iam_file.columns)):
                        self.__trigger_mail(user=user)
        if append_data:
            self.__google_drive_operations.append_values(spreadsheet_id=self.__SPREADSHEET_ID, sheet_name=self.__sheet_name, values=append_data)

    def __trigger_mail(self, user: str):
        """
        This method send mail
        @param user:
        @return:
        """
        special_user_mails = self.__literal_eval()
        to = user if user not in special_user_mails else special_user_mails[user]
        ldap_data = self.__ldap.get_user_details(user_name=to)
        cc = [os.environ.get("account_admin", '')]
        name = to
        if ldap_data:
            cc.append(f'{ldap_data.get("managerId")}@redhat.com')
            name = ldap_data.get('displayName')
        subject, body = MailMessage().iam_user_add_tags(name=name, user=user, spreadsheet_id=self.__SPREADSHEET_ID)
        self.__mail.send_email_postfix(to=to, content=body, subject=subject, cc=cc)



import csv
import os.path

import google.auth
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

from cloud_governance.common.logger.init_logger import logger


class GoogleDriveOperations:
    """
    This class perform the Google Drive Operations
    methods
    1. download_spreadsheet
    """

    def __init__(self):
        self.__creds, _ = google.auth.default()
        self.__service = build('sheets', 'v4', credentials=self.__creds)

    def download_spreadsheet(self, spreadsheet_id: str, sheet_name: str, file_path: str):
        """
        This method download spreadsheet from the Google Drive
        Used the Google Drive API
        Create GCP Project, enable Google Drive API, enable Google Spreadsheet API
        Create Service Account, and generate keys in json format
        export GOOGLE_APPLICATION_CREDENTIALS=file_location
        @return:
        """
        try:
            result = self.__service.spreadsheets().values().get(spreadsheetId=spreadsheet_id, range=sheet_name).execute()
            file_name = f'{sheet_name}.csv'
            output_file = os.path.join(file_path, file_name)
            with open(output_file, 'w') as f:
                writer = csv.writer(f)
                writer.writerows(result.get('values'))
        except HttpError as error:
            logger.info(f'An error occurred: {error}')
        logger.info(f'Successfully downloaded {sheet_name}.csv')

    def append_values(self, spreadsheet_id, sheet_name: str, values: list, value_input_option: str = 'USER_ENTERED'):
        """
        This method append the values in the spreadsheet
        @param spreadsheet_id:
        @param sheet_name:
        @param values:
        @param value_input_option:
        @return:
        """
        try:
            body = {'values': values}
            result = self.__service.spreadsheets().values().append(
                spreadsheetId=spreadsheet_id, range=sheet_name,
                valueInputOption=value_input_option, body=body).execute()
            return result
        except HttpError as error:
            logger.info(f'An error occurred: {error}')

    def find_sheet_id_by_name(self, sheet_name: str, spreadsheet_id: str):
        """
        This method find the sheet id in the spreadsheet
        @param sheet_name:
        @param spreadsheet_id:
        @return:
        """
        sheets_with_properties = self.__service.spreadsheets() .get(spreadsheetId=spreadsheet_id, fields='sheets.properties').execute().get('sheets')
        for sheet in sheets_with_properties:
            if 'title' in sheet['properties'].keys():
                if sheet['properties']['title'] == sheet_name:
                    return sheet['properties']['sheetId']

    def delete_rows(self, spreadsheet_id: str, sheet_name: str, row_number: int):
        """
        This method delete row from the spreadsheet bases on row number
        @param spreadsheet_id:
        @param sheet_name:
        @param row_number:
        @return:
        """
        try:
            spreadsheet_data = [
                {
                    "deleteDimension": {
                        "range": {
                            "sheetId": self.find_sheet_id_by_name(sheet_name=sheet_name, spreadsheet_id=spreadsheet_id),
                            "dimension": "ROWS",
                            "startIndex": row_number,
                            "endIndex": row_number+1
                        }
                    }
                }
            ]
            update_data = {"requests": spreadsheet_data}
            updating = self.__service.spreadsheets().batchUpdate(spreadsheetId=spreadsheet_id, body=update_data)
            updating.execute()
        except HttpError as error:
            logger.into(f'An error occurred: {error}')

    def paste_csv_to_gsheet(self, csv_path, spreadsheet_id: str, sheet_name: str):
        """
        This method paste the csv data into the specific sheet
        Note: replaces the content in the rowIndex, columnIndex you specified
        @param csv_path:
        @param spreadsheet_id:
        @param sheet_name:
        @return:
        """
        csv_contents = None
        with open(csv_path, 'r') as csv_file:
            csv_contents = csv_file.read()
        if csv_contents:
            sheet_id = self.find_sheet_id_by_name(sheet_name=sheet_name, spreadsheet_id=spreadsheet_id)
            body = {
                'requests': [{
                    'pasteData': {
                        "coordinate": {
                            "sheetId": sheet_id,
                            "rowIndex": "0",
                            "columnIndex": "0",
                        },
                        "data": csv_contents,
                        "type": 'PASTE_NORMAL',
                        "delimiter": ',',
                    }
                }]
            }
            request = self.__service.spreadsheets().batchUpdate(spreadsheetId=spreadsheet_id, body=body)
            response = request.execute()
            return response



import os


class MailMessage:

    RESTRICTION = 'Don\'t replay this email, in case of issue update us in slack channel #perf-dept-public-clouds.'

    def __init__(self):
        self.account = os.environ.get('account', '')
        self.region = os.environ.get('AWS_DEFAULT_REGION', '')

    def ec2_stop(self, name: str, days: int, image_id: str, delete_instance_days: int, instance_name: str, resource_id: str, stopped_time: str):
        subject = f'cloud-governance alert: delete ec2-stop more than {days} days'
        content = 'This instance will be deleted after 30 days.\nPlease add Policy=Not_Delete to your tags for skipping this policy. If you already added ignore this email.'
        message = ''
        if image_id != '':
            content = f'You can find a image of the deleted image under AMI: {image_id}'
            message = f'This instance will be deleted due to it was stopped more than {delete_instance_days} days.'
        body = f"""
Greetings {name},

Instance: {instance_name}: {resource_id} in {self.region} on AWS account: {self.account} was stopped on {stopped_time}, it stopped more than {days} days.  
{message}
{content}

{self.RESTRICTION}

Best regards,
Cloud-governance Team""".strip()
        return subject, body

    def ec2_idle(self, name: str, days: int, notification_days: int, stop_days: int, instance_name: str, resource_id: str):
        if days == notification_days:
            subject = f'cloud-governance alert:  ec2-idle is {notification_days} days'
            cause = f'This instance will be stopped if it is idle {stop_days} days'
            content = 'If you want that cloud-governance will not stop it add Policy=Not_Delete tag to your instance. '
        else:
            subject = f'cloud-governance alert: Stopped ec2-idle more than {stop_days} days'
            cause = f'This instance will be stopped.'
            content = 'In future cloud-governance will not stop it add Policy=Not_Delete tag to your instance'
        body = f"""
Greetings {name},

Instance: {instance_name}: {resource_id} in {self.region} on AWS account: {self.account} is idle more than {days} days.
{cause}
{content}
If you already added the Policy=Not_Delete tag ignore this mail.

{self.RESTRICTION}

Best Regards,
Cloud-governance Team""".strip()
        return subject, body

    def iam_user_add_tags(self, name: str, user: str, spreadsheet_id: str):
        subject = f'cloud-governance alert: Missing tags in AWS IAM User'
        body = f"""
Hi {name},

{os.environ.get('account')} IAM User: {user} has missing tags 
Please add the tags in the spreadsheet: https://docs.google.com/spreadsheets/d/{spreadsheet_id}.
If you already filled the tags, please ignore the mail.

{self.RESTRICTION}

Best Regards
Cloud-governance Team""".strip()
        return subject, body

    def aws_user_over_usage_cost(self, user: str, usage_cost: int, name: str, user_usage: int):
        """
        This method send subject, body to over usage cost
        @return:
        """
        subject = f'cloud-governance alert: last week cost overage {usage_cost}$ per user'
        body = f"""
Hi {name},
Your AWS user:{user} in account:{os.environ.get('account')} cost was {user_usage}$ in the last week. 
Please verify that you are using all the resources.

{self.RESTRICTION}

Best Regards,
Cloud-governance Team""".strip()
        return subject, body



from cloud_governance.common.logger.init_logger import logger
from cloud_governance.aws.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy


class ZombieSnapshots(NonClusterZombiePolicy):

    def __init__(self):
        super().__init__()

    def _get_image_ids_from_description(self, snapshot_description: str):
        """
        This method gets image Ids from snapshot description
        Two cases:
        # Created by CreateImage(i-******) for ami-********
        # Copied for DestinationAmi ami-******* from SourceAmi ami-******* for SourceSnapshot snap-******. Task created on 1,566,308,778,174.
        @return:
        """
        image_ids = []
        images_array = snapshot_description.split('ami-')[1:]
        for image in images_array:
            image_ids.append(f'ami-{image.split(" ")[0]}')
        return image_ids

    def run(self):
        """
        This method return all the zombie snapshots, delete if dry_run no
        @return:
        """
        snapshots = self._ec2_operations.get_snapshots()
        zombie_snapshots = {}
        zombie_snapshots_data = []
        image_ids = self._get_ami_ids()
        for snapshot in snapshots:
            if snapshot.get('Description'):
                snapshot_images = self._get_image_ids_from_description(snapshot.get('Description'))
                found = False
                for snapshot_image in snapshot_images:
                    if snapshot_image in image_ids:
                        found = True
                if not found:
                    zombie_snapshots[snapshot.get('SnapshotId')] = snapshot.get('Tags')
                    zombie_snapshots_data.append([snapshot.get('SnapshotId'),
                                                  self._get_tag_name_from_tags(tags=snapshot.get('Tags')),
                                                  self._get_tag_name_from_tags(tags=snapshot.get('Tags'), tag_name='User'),
                                                  str(snapshot.get('VolumeSize')),
                                                  self._get_policy_value(tags=snapshot.get('Tags'))
                                                  ])
        if self._dry_run == "no":
            for zombie_id, tags in zombie_snapshots.items():
                if self._get_policy_value(tags=tags) != 'NOTDELETE':
                    self._ec2_client.delete_snapshot(SnapshotId=zombie_id)
                    logger.info(f'Snapshot is deleted {zombie_id}')
        return zombie_snapshots_data



import os
import smtplib
from email.message import EmailMessage

from cloud_governance.common.logger.init_logger import logger

# https://github.com/redhat-performance/quads/blob/master/quads/tools/postman.py


class Postfix:
    """
    This class send mail using the postfix configuration
    install postfix and configure on machine
    $ dnf install postfix -y
    $ vi /etc/postfix/main.cf
    relayhost = [smtp.corp.redhat.com]
    inet_interfaces = all
    inet_protocols = all
    relay_domains = rdu2.scalelab.redhat.com, redhat.com
    $ systemctl restart postfix
    """

    def __init__(self):
        self.reply_to = os.environ.get('REPLY_TO', 'dev-null@redhat.com')

    def send_email_postfix(self, subject: str, to: str, cc: list, content: str):
        msg = EmailMessage()
        msg["Subject"] = subject
        msg["From"] = "%s <%s>" % (
            'cloud-governance',
            "@".join(["noreply-cloud-governance", 'redhat.com']),
        )
        msg["To"] = "@".join([to, 'redhat.com'])
        msg["Cc"] = ",".join(cc)
        # msg.add_header("Reply-To", self.reply_to)
        # msg.add_header("User-Agent", self.reply_to)
        msg.set_content(content)
        email_string = msg.as_string()
        email_host = 'localhost'
        try:
            with smtplib.SMTP(email_host) as s:
                try:
                    logger.debug(email_string)
                    s.send_message(msg)
                    logger.info(f'Mail sent successfully to {to}@redhat.com')
                except smtplib.SMTPException as ex:
                    logger.info(f'Error while sending mail, {ex}')
                    return False
            return True
        except Exception as err:
            logger.info(f'Some error occurred, {err}')
            return False



from cloud_governance.common.logger.init_logger import logger
from cloud_governance.aws.zombie_non_cluster.run_zombie_non_cluster_policies import NonClusterZombiePolicy


class EmptyRoles(NonClusterZombiePolicy):

    def __init__(self):
        super().__init__()

    def run(self):
        """
        This method return all empty roles, delete if dry_run no
        @return:
        """
        zombie_roles_data = []
        zombie_roles = {}
        roles = self._iam_operations.get_roles()
        active_clusters = self._zombie_cluster.all_cluster_instance()
        for role in roles:
            try:
                role_attached_policies = self._iam_client.list_attached_role_policies(RoleName=role.get('RoleName'))
                role_inline_policies = self._iam_client.list_role_policies(RoleName=role.get('RoleName'))
                if not role_inline_policies.get('PolicyNames') and not role_attached_policies.get('AttachedPolicies'):
                    get_role = self._iam_client.get_role(RoleName=role.get('RoleName'))['Role']
                    if not self._check_live_cluster_tag(get_role.get('Tags'), active_clusters.values()):
                        zombie_roles_data.append([role.get('RoleName'), self._get_policy_value(tags=get_role.get('Tags'))])
                        zombie_roles[role.get('RoleName')] = get_role.get('Tags')
            except Exception as err:
                logger.info(f'{err} {role.get("RoleName")}')
        if self._dry_run == 'no':
            for zombie_role, tags in zombie_roles.items():
                if self._get_policy_value(tags=tags) != 'NOTDELETE':
                    try:
                        self._iam_client.delete_role(RoleName=zombie_role)
                        logger.info(f'Role is deleted: {zombie_role}')
                    except Exception as err:
                        logger.info(f'Exception raised: {err}: {zombie_role}')
        return zombie_roles_data



import os

from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.common.ldap.ldap_search import LdapSearch


class CostOverUsage(ElasticUpload):

    FETCH_DAYS = 7
    COST_USAGE_DOLLAR = 1000

    def __init__(self):
        super().__init__()
        self.__ldap_host_name = os.environ.get('LDAP_HOST_NAME', '')
        self.__ldap = LdapSearch(ldap_host_name=self.__ldap_host_name)

    def aws_user_usage(self, days: int, cost_usage: int):
        """
        This method send mail when cost_usage is greater than given cost usage in last specified days
        @param cost_usage:
        @param days:
        @return:
        """
        users = []
        cc = []
        user_data = self._elastic_search_operations.get_index_hits(days=days, index=self._es_index)
        for user_usage in user_data:
            user = user_usage['User']
            if user_usage['Cost'] > cost_usage:
                special_user_mails = self._literal_eval(self._special_user_mails)
                to = user if user not in special_user_mails else special_user_mails[user]
                ldap_data = self.__ldap.get_user_details(user_name=to)
                name = to
                if ldap_data:
                    cc.append(f'{ldap_data.get("managerId")}@redhat.com')
                    name = ldap_data.get('displayName')
                subject, body = self._mail_message.aws_user_over_usage_cost(user=to, user_usage=user_usage['Cost'], name=name, usage_cost=self.COST_USAGE_DOLLAR)
                self._postfix_mail.send_email_postfix(subject=subject, content=body, to=to, cc=cc)
                users.append(to)
        return users

    def run(self):
        """
        This method runs the cost usage
        @return:
        """
        return self.aws_user_usage(days=self.FETCH_DAYS, cost_usage=self.COST_USAGE_DOLLAR)



import os
from datetime import datetime, timedelta

import boto3

from cloud_governance.common.clouds.aws.cloudtrail.cloudtrail_operations import CloudTrailOperations
from cloud_governance.common.clouds.aws.dynamodb.dynamodb_operations import DynamoDbOperations
from cloud_governance.common.logger.init_logger import logger


class UploadDataToDynamoDb:
    """
    This class is upload cloudtrail data to DynamoDb
    """

    def __init__(self):
        self._region = os.environ.get('AWS_DEFAULT_REGION', 'ap-northeast-1')
        self._db_client = boto3.client('dynamodb', region_name=self._region)
        self._db_operations = DynamoDbOperations(region_name=self._region)
        self._table_name = os.environ.get('TABLE_NAME', 'test_data')
        self._end_time = datetime.now() - timedelta(days=0)
        self._start_time = self._end_time - timedelta(days=1)
        self._cloudtrail_operations = CloudTrailOperations(region_name=self._region)

    def __convert_datatime_to_timestamp_in_data(self, data: list):
        """
        This method convert datetime to timestamp
        @param data:
        @return:
        """
        for item in data:
            if item.get('EventTime'):
                item['EventTime'] = round(item.get('EventTime').timestamp())
        return data

    def _create_table_not_exists(self, primary_key: str):
        """
        This method create table if not exists
        @return:
        """
        try:
            self._db_client.describe_table(TableName=self._table_name)
        except self._db_client.exceptions.ResourceNotFoundException:
            self._db_operations.create_table(table_name=self._table_name, key_name=primary_key)
            logger.info(f'Table is created {self._table_name}')

    def _upload_to_dynamo_db_table(self, data: list):
        """
        This method upload data to DynamoDB
        @param data:
        @return:
        """
        data = self.__convert_datatime_to_timestamp_in_data(data)
        count = 0
        for item in data:
            self._db_operations.put_item(table_name=self._table_name, item=self._db_operations.serialize_data_dynamodb_data(item=item))
            count += 1
        return count
